@@ -1,24 +1,18 @@
 /*
-  +---------------------------------------------------------------------------+
-  | PHP Driver for MongoDB                                                    |
-  +---------------------------------------------------------------------------+
-  | Copyright 2013-2015 MongoDB, Inc.                                         |
-  |                                                                           |
-  | Licensed under the Apache License, Version 2.0 (the "License");           |
-  | you may not use this file except in compliance with the License.          |
-  | You may obtain a copy of the License at                                   |
-  |                                                                           |
-  | http://www.apache.org/licenses/LICENSE-2.0                                |
-  |                                                                           |
-  | Unless required by applicable law or agreed to in writing, software       |
-  | distributed under the License is distributed on an "AS IS" BASIS,         |
-  | WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  |
-  | See the License for the specific language governing permissions and       |
-  | limitations under the License.                                            |
-  +---------------------------------------------------------------------------+
-  | Copyright (c) 2014-2015 MongoDB, Inc.                                     |
-  +---------------------------------------------------------------------------+
-*/
+ * Copyright 2014-2017 MongoDB, Inc.
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *   http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 #ifdef HAVE_CONFIG_H
 #	include "config.h"
@@ -79,6 +73,10 @@ ZEND_DECLARE_MODULE_GLOBALS(mongodb)
 #endif
 #endif
 
+/* Declare zend_class_entry dependencies, which are initialized in MINIT */
+zend_class_entry *php_phongo_date_immutable_ce;
+zend_class_entry *php_phongo_json_serializable_ce;
+
 php_phongo_server_description_type_map_t
 php_phongo_server_description_type_map[PHONGO_SERVER_DESCRIPTION_TYPES] = {
 	{ PHONGO_SERVER_UNKNOWN, "Unknown" },
@@ -354,35 +352,35 @@ void phongo_writeconcern_init(zval *return_value, const mongoc_write_concern_t *
 zend_bool phongo_writeconcernerror_init(zval *return_value, bson_t *bson TSRMLS_DC) /* {{{ */
 {
 	bson_iter_t iter;
-	php_phongo_writeconcernerror_t *writeconcernerror;
+	php_phongo_writeconcernerror_t *intern;
 
-	writeconcernerror = Z_WRITECONCERNERROR_OBJ_P(return_value);
+	object_init_ex(return_value, php_phongo_writeconcernerror_ce);
+
+	intern = Z_WRITECONCERNERROR_OBJ_P(return_value);
 
 	if (bson_iter_init_find(&iter, bson, "code") && BSON_ITER_HOLDS_INT32(&iter)) {
-		writeconcernerror->code = bson_iter_int32(&iter);
+		intern->code = bson_iter_int32(&iter);
 	}
+
 	if (bson_iter_init_find(&iter, bson, "errmsg") && BSON_ITER_HOLDS_UTF8(&iter)) {
 		uint32_t errmsg_len;
 		const char *err_msg = bson_iter_utf8(&iter, &errmsg_len);
 
-		writeconcernerror->message = estrndup(err_msg, errmsg_len);
+		intern->message = estrndup(err_msg, errmsg_len);
 	}
+
 	if (bson_iter_init_find(&iter, bson, "errInfo") && BSON_ITER_HOLDS_DOCUMENT(&iter)) {
-		uint32_t               len;
-		const uint8_t         *data;
+		uint32_t       len;
+		const uint8_t *data = NULL;
 
 		bson_iter_document(&iter, &len, &data);
 
-		if (!data) {
-			return false;
-		}
-
-		if (!phongo_bson_to_zval(data, len, &writeconcernerror->info)) {
-			zval_ptr_dtor(&writeconcernerror->info);
+		if (!phongo_bson_to_zval(data, len, &intern->info)) {
+			zval_ptr_dtor(&intern->info);
 #if PHP_VERSION_ID >= 70000
-			ZVAL_UNDEF(&writeconcernerror->info);
+			ZVAL_UNDEF(&intern->info);
 #else
-			writeconcernerror->info = NULL;
+			intern->info = NULL;
 #endif
 
 			return false;
@@ -395,38 +393,43 @@ zend_bool phongo_writeconcernerror_init(zval *return_value, bson_t *bson TSRMLS_
 zend_bool phongo_writeerror_init(zval *return_value, bson_t *bson TSRMLS_DC) /* {{{ */
 {
 	bson_iter_t iter;
-	php_phongo_writeerror_t *writeerror;
+	php_phongo_writeerror_t *intern;
 
-	writeerror = Z_WRITEERROR_OBJ_P(return_value);
+	object_init_ex(return_value, php_phongo_writeerror_ce);
+
+	intern = Z_WRITEERROR_OBJ_P(return_value);
 
 	if (bson_iter_init_find(&iter, bson, "code") && BSON_ITER_HOLDS_INT32(&iter)) {
-		writeerror->code = bson_iter_int32(&iter);
+		intern->code = bson_iter_int32(&iter);
 	}
+
 	if (bson_iter_init_find(&iter, bson, "errmsg") && BSON_ITER_HOLDS_UTF8(&iter)) {
 		uint32_t errmsg_len;
 		const char *err_msg = bson_iter_utf8(&iter, &errmsg_len);
 
-		writeerror->message = estrndup(err_msg, errmsg_len);
+		intern->message = estrndup(err_msg, errmsg_len);
 	}
-	if (bson_iter_init_find(&iter, bson, "errInfo")) {
-		bson_t                 info;
 
-		bson_init(&info);
-		bson_append_iter(&info, NULL, 0, &iter);
+	if (bson_iter_init_find(&iter, bson, "errInfo") && BSON_ITER_HOLDS_DOCUMENT(&iter)) {
+		uint32_t       len;
+		const uint8_t *data = NULL;
 
-		if (!phongo_bson_to_zval(bson_get_data(&info), info.len, &writeerror->info)) {
-			zval_ptr_dtor(&writeerror->info);
+		bson_iter_document(&iter, &len, &data);
+
+		if (!phongo_bson_to_zval(data, len, &intern->info)) {
+			zval_ptr_dtor(&intern->info);
 #if PHP_VERSION_ID >= 70000
-			ZVAL_UNDEF(&writeerror->info);
+			ZVAL_UNDEF(&intern->info);
 #else
-			writeerror->info = NULL;
+			intern->info = NULL;
 #endif
 
 			return false;
 		}
 	}
+
 	if (bson_iter_init_find(&iter, bson, "index") && BSON_ITER_HOLDS_INT32(&iter)) {
-		writeerror->index = bson_iter_int32(&iter);
+		intern->index = bson_iter_int32(&iter);
 	}
 
 	return true;
@@ -837,7 +840,9 @@ void php_phongo_read_preference_prep_tagsets(zval *tagSets TSRMLS_DC) /* {{{ */
 		zval *tagSet;
 
 		ZEND_HASH_FOREACH_VAL(ht_data, tagSet) {
+			ZVAL_DEREF(tagSet);
 			if (Z_TYPE_P(tagSet) == IS_ARRAY) {
+				SEPARATE_ZVAL_NOREF(tagSet);
 				convert_to_object(tagSet);
 			}
 		} ZEND_HASH_FOREACH_END();
@@ -851,6 +856,7 @@ void php_phongo_read_preference_prep_tagsets(zval *tagSets TSRMLS_DC) /* {{{ */
 		     zend_hash_get_current_data_ex(ht_data, (void **) &tagSet, &pos) == SUCCESS;
 		     zend_hash_move_forward_ex(ht_data, &pos)) {
 			if (Z_TYPE_PP(tagSet) == IS_ARRAY) {
+				SEPARATE_ZVAL_IF_NOT_REF(tagSet);
 				convert_to_object(*tagSet);
 			}
 		}
@@ -875,7 +881,7 @@ bool php_phongo_read_preference_tags_are_valid(const bson_t *tags) /* {{{ */
 	}
 
 	while (bson_iter_next(&iter)) {
-		if (!BSON_ITER_HOLDS_DOCUMENT(&iter) && !BSON_ITER_HOLDS_ARRAY(&iter)) {
+		if (!BSON_ITER_HOLDS_DOCUMENT(&iter)) {
 			return false;
 		}
 	}
@@ -1511,8 +1517,7 @@ void phongo_manager_init(php_phongo_manager_t *manager, const char *uri_string,
 	bson_iter_t       iter;
 
 #if PHP_VERSION_ID >= 70000
-	zval             *client_ptr;
-	zval              new_client_ptr;
+	mongoc_client_t  *client_ptr;
 #else
 	mongoc_client_t **client_ptr;
 #endif
@@ -1523,9 +1528,9 @@ void phongo_manager_init(php_phongo_manager_t *manager, const char *uri_string,
 	}
 
 #if PHP_VERSION_ID >= 70000
-	if ((client_ptr = zend_hash_str_find(&MONGODB_G(clients), hash, hash_len)) && Z_TYPE_P(client_ptr) == IS_PTR) {
+	if ((client_ptr = zend_hash_str_find_ptr(&MONGODB_G(clients), hash, hash_len)) != NULL) {
 		MONGOC_DEBUG("Found client for hash: %s\n", hash);
-		manager->client = (mongoc_client_t *)Z_PTR_P(client_ptr);
+		manager->client = client_ptr;
 		goto cleanup;
 	}
 #else
@@ -1582,8 +1587,7 @@ void phongo_manager_init(php_phongo_manager_t *manager, const char *uri_string,
 
 	MONGOC_DEBUG("Created client hash: %s\n", hash);
 #if PHP_VERSION_ID >= 70000
-	ZVAL_PTR(&new_client_ptr, manager->client);
-	zend_hash_str_update(&MONGODB_G(clients), hash, hash_len, &new_client_ptr);
+	zend_hash_str_update_ptr(&MONGODB_G(clients), hash, hash_len, manager->client);
 #else
 	zend_hash_update(&MONGODB_G(clients), hash, hash_len + 1, &manager->client, sizeof(mongoc_client_t *), NULL);
 #endif
@@ -1666,6 +1670,15 @@ void php_phongo_new_decimal128(zval *object, const bson_decimal128_t *decimal TS
 	intern->initialized = true;
 } /* }}} */
 
+/* qsort() compare callback for alphabetizing regex flags upon initialization */
+static int php_phongo_regex_compare_flags(const void *f1, const void *f2) {
+	if (* (const char *) f1 == * (const char *) f2) {
+		return 0;
+	}
+
+	return (* (const char *) f1 > * (const char *) f2) ? 1 : -1;
+}
+
 void php_phongo_new_regex_from_regex_and_options(zval *object, const char *pattern, const char *flags TSRMLS_DC) /* {{{ */
 {
 	php_phongo_regex_t     *intern;
@@ -1677,184 +1690,12 @@ void php_phongo_new_regex_from_regex_and_options(zval *object, const char *patte
 	intern->pattern = estrndup(pattern, intern->pattern_len);
 	intern->flags_len = strlen(flags);
 	intern->flags = estrndup(flags, intern->flags_len);
+
+	/* Ensure flags are alphabetized upon initialization. This may be removed
+	 * once CDRIVER-1883 is implemented. */
+	qsort((void *) intern->flags, intern->flags_len, 1, php_phongo_regex_compare_flags);
 } /* }}} */
 
-static void php_phongo_cursor_free_current(php_phongo_cursor_t *cursor) /* {{{ */
-{
-	if (!Z_ISUNDEF(cursor->visitor_data.zchild)) {
-		zval_ptr_dtor(&cursor->visitor_data.zchild);
-#if PHP_VERSION_ID >= 70000
-		ZVAL_UNDEF(&cursor->visitor_data.zchild);
-#else
-		cursor->visitor_data.zchild = NULL;
-#endif
-	}
-} /* }}} */
-
-void php_phongo_cursor_free(php_phongo_cursor_t *cursor) /* {{{ */
-{
-	if (cursor->cursor) {
-		mongoc_cursor_destroy(cursor->cursor);
-		cursor->cursor = NULL;
-	}
-
-	php_phongo_cursor_free_current(cursor);
-} /* }}} */
-
-/* {{{ Iterator */
-static void php_phongo_cursor_iterator_dtor(zend_object_iterator *iter TSRMLS_DC) /* {{{ */
-{
-	php_phongo_cursor_iterator *cursor_it = (php_phongo_cursor_iterator *)iter;
-
-	if (!Z_ISUNDEF(cursor_it->intern.data)) {
-#if PHP_VERSION_ID >= 70000
-		zval_ptr_dtor(&cursor_it->intern.data);
-#else
-		zval_ptr_dtor((zval**)&cursor_it->intern.data);
-		cursor_it->intern.data = NULL;
-#endif
-	}
-
-#if PHP_VERSION_ID < 70000
-	efree(cursor_it);
-#endif
-} /* }}} */
-
-static int php_phongo_cursor_iterator_valid(zend_object_iterator *iter TSRMLS_DC) /* {{{ */
-{
-	php_phongo_cursor_t *cursor = ((php_phongo_cursor_iterator *)iter)->cursor;
-
-	if (!Z_ISUNDEF(cursor->visitor_data.zchild)) {
-		return SUCCESS;
-	}
-
-	return FAILURE;
-} /* }}} */
-
-#if PHP_VERSION_ID < 50500
-static int php_phongo_cursor_iterator_get_current_key(zend_object_iterator *iter, char **str_key, uint *str_key_len, ulong *int_key TSRMLS_DC) /* {{{ */
-{
-	php_phongo_cursor_t *cursor = ((php_phongo_cursor_iterator *)iter)->cursor;
-
-	*int_key = (ulong) cursor->current;
-	return HASH_KEY_IS_LONG;
-} /* }}} */
-#else
-static void php_phongo_cursor_iterator_get_current_key(zend_object_iterator *iter, zval *key TSRMLS_DC) /* {{{ */
-{
-	php_phongo_cursor_t *cursor = ((php_phongo_cursor_iterator *)iter)->cursor;
-
-	ZVAL_LONG(key, cursor->current);
-} /* }}} */
-#endif
-
-#if PHP_VERSION_ID < 70000
-static void php_phongo_cursor_iterator_get_current_data(zend_object_iterator *iter, zval ***data TSRMLS_DC) /* {{{ */
-{
-	php_phongo_cursor_t *cursor = ((php_phongo_cursor_iterator *)iter)->cursor;
-
-	*data = &cursor->visitor_data.zchild;
-} /* }}} */
-#else
-static zval* php_phongo_cursor_iterator_get_current_data(zend_object_iterator *iter) /* {{{ */
-{
-	php_phongo_cursor_t *cursor = ((php_phongo_cursor_iterator *)iter)->cursor;
-
-	return &cursor->visitor_data.zchild;
-} /* }}} */
-#endif
-
-static void php_phongo_cursor_iterator_move_forward(zend_object_iterator *iter TSRMLS_DC) /* {{{ */
-{
-	php_phongo_cursor_iterator *cursor_it = (php_phongo_cursor_iterator *)iter;
-	php_phongo_cursor_t        *cursor = cursor_it->cursor;
-	const bson_t               *doc;
-
-	php_phongo_cursor_free_current(cursor);
-	cursor->current++;
-
-	if (mongoc_cursor_next(cursor->cursor, &doc)) {
-		phongo_bson_to_zval_ex(bson_get_data(doc), doc->len, &cursor->visitor_data);
-	} else {
-		bson_error_t error;
-
-		if (mongoc_cursor_error(cursor->cursor, &error)) {
-			/* Intentionally not destroying the cursor as it will happen
-			 * naturally now that there are no more results */
-			phongo_throw_exception_from_bson_error_t(&error TSRMLS_CC);
-		}
-	}
-} /* }}} */
-
-static void php_phongo_cursor_iterator_rewind(zend_object_iterator *iter TSRMLS_DC) /* {{{ */
-{
-	php_phongo_cursor_iterator *cursor_it = (php_phongo_cursor_iterator *)iter;
-	php_phongo_cursor_t        *cursor = cursor_it->cursor;
-	const bson_t               *doc;
-
-	if (cursor->current > 0) {
-		phongo_throw_exception(PHONGO_ERROR_LOGIC TSRMLS_CC, "Cursors cannot rewind after starting iteration");
-		return;
-	}
-
-	php_phongo_cursor_free_current(cursor);
-
-	doc = mongoc_cursor_current(cursor->cursor);
-
-	if (doc) {
-		phongo_bson_to_zval_ex(bson_get_data(doc), doc->len, &cursor->visitor_data);
-	}
-} /* }}} */
-
-/* iterator handler table */
-zend_object_iterator_funcs php_phongo_cursor_iterator_funcs = {
-	php_phongo_cursor_iterator_dtor,
-	php_phongo_cursor_iterator_valid,
-	php_phongo_cursor_iterator_get_current_data,
-	php_phongo_cursor_iterator_get_current_key,
-	php_phongo_cursor_iterator_move_forward,
-	php_phongo_cursor_iterator_rewind,
-	NULL /* invalidate_current is not used */
-};
-
-zend_object_iterator *php_phongo_cursor_get_iterator(zend_class_entry *ce, zval *object, int by_ref TSRMLS_DC) /* {{{ */
-{
-	php_phongo_cursor_iterator *cursor_it = NULL;
-	php_phongo_cursor_t *cursor = Z_CURSOR_OBJ_P(object);
-
-	if (by_ref) {
-		zend_error(E_ERROR, "An iterator cannot be used with foreach by reference");
-	}
-
-	if (cursor->got_iterator) {
-		phongo_throw_exception(PHONGO_ERROR_LOGIC TSRMLS_CC, "Cursors cannot yield multiple iterators");
-		return NULL;
-	}
-
-	cursor->got_iterator = 1;
-
-	cursor_it = ecalloc(1, sizeof(php_phongo_cursor_iterator));
-#if PHP_VERSION_ID >= 70000
-	zend_iterator_init(&cursor_it->intern);
-#endif
-
-#if PHP_VERSION_ID >= 70000
-	ZVAL_COPY(&cursor_it->intern.data, object);
-#else
-	Z_ADDREF_P(object);
-	cursor_it->intern.data  = (void*)object;
-#endif
-	cursor_it->intern.funcs = &php_phongo_cursor_iterator_funcs;
-	cursor_it->cursor = cursor;
-	/* cursor_it->current should already be allocated to zero */
-
-	php_phongo_cursor_free_current(cursor_it->cursor);
-
-	return &cursor_it->intern;
-} /* }}} */
-/* }}} */
-
-
 /* {{{ Memory allocation wrappers */
 static void* php_phongo_malloc(size_t num_bytes) /* {{{ */
 {
@@ -1984,6 +1825,18 @@ PHP_INI_BEGIN()
 PHP_INI_END()
 /* }}} */
 
+#if PHP_VERSION_ID >= 70000
+static void php_phongo_client_dtor(zval *zv)
+{
+	mongoc_client_destroy((mongoc_client_t *) Z_PTR_P(zv));
+}
+#else
+static void php_phongo_client_dtor(void *client)
+{
+	mongoc_client_destroy(*((mongoc_client_t **) client));
+}
+#endif
+
 /* {{{ PHP_GINIT_FUNCTION */
 PHP_GINIT_FUNCTION(mongodb)
 {
@@ -1998,24 +1851,32 @@ PHP_GINIT_FUNCTION(mongodb)
 	ZEND_TSRMLS_CACHE_UPDATE();
 #endif
 #endif
-	mongodb_globals->debug_fd = NULL;
+	memset(mongodb_globals, 0, sizeof(zend_mongodb_globals));
 	mongodb_globals->bsonMemVTable = bsonMemVTable;
-
+	/* Initialize HashTable for persistent clients */
+	zend_hash_init_ex(&mongodb_globals->clients, 0, NULL, php_phongo_client_dtor, 1, 0);
 }
 /* }}} */
 
+static zend_class_entry *php_phongo_fetch_internal_class(const char *class_name, size_t class_name_len TSRMLS_DC)
+{
 #if PHP_VERSION_ID >= 70000
-static void php_phongo_client_dtor(zval *zv)
-{
-	mongoc_client_destroy((mongoc_client_t *) Z_PTR_P(zv));
-}
+	zend_class_entry *pce;
+
+	if ((pce = zend_hash_str_find_ptr(CG(class_table), class_name, class_name_len))) {
+		return pce;
+	}
 #else
-static void php_phongo_client_dtor(void *client)
-{
-	mongoc_client_destroy(*((mongoc_client_t **) client));
-}
+	zend_class_entry **pce;
+
+	if (zend_hash_find(CG(class_table), class_name, class_name_len + 1, (void **) &pce) == SUCCESS) {
+		return *pce;
+	}
 #endif
 
+	return NULL;
+}
+
 /* {{{ PHP_MINIT_FUNCTION */
 PHP_MINIT_FUNCTION(mongodb)
 {
@@ -2037,9 +1898,6 @@ PHP_MINIT_FUNCTION(mongodb)
 	/* Initialize libbson */
 	bson_mem_set_vtable(&MONGODB_G(bsonMemVTable));
 
-	/* Initialize HashTable for persistent clients */
-	zend_hash_init(&MONGODB_G(clients), 0, NULL, php_phongo_client_dtor, 1);
-
 	/* Prep default object handlers to be used when we register the classes */
 	memcpy(&phongo_std_object_handlers, zend_get_std_object_handlers(), sizeof(zend_object_handlers));
 	phongo_std_object_handlers.clone_obj            = NULL;
@@ -2051,6 +1909,25 @@ PHP_MINIT_FUNCTION(mongodb)
 	phongo_std_object_handlers.get_closure          = NULL;
 	*/
 
+	/* Initialize zend_class_entry dependencies.
+	 *
+	 * Although DateTimeImmutable was introduced in PHP 5.5.0,
+	 * php_date_get_immutable_ce() is not available in PHP versions before
+	 * 5.5.24 and 5.6.8.
+	 *
+	 * Although JsonSerializable was introduced in PHP 5.4.0,
+	 * php_json_serializable_ce is not exported in PHP versions before 5.4.26
+	 * and 5.5.10. For later PHP versions, looking up the class manually also
+	 * helps with distros that disable LTDL_LAZY for dlopen() (e.g. Fedora).
+	 */
+	php_phongo_date_immutable_ce = php_phongo_fetch_internal_class(ZEND_STRL("datetimeimmutable") TSRMLS_CC);
+	php_phongo_json_serializable_ce = php_phongo_fetch_internal_class(ZEND_STRL("jsonserializable") TSRMLS_CC);
+
+	if (php_phongo_json_serializable_ce == NULL) {
+		zend_error(E_ERROR, "JsonSerializable class is not defined. Please ensure that the 'json' module is loaded before the 'mongodb' module.");
+		return FAILURE;
+	}
+
 	PHP_MINIT(bson)(INIT_FUNC_ARGS_PASSTHRU);
 
 	PHP_MINIT(Type)(INIT_FUNC_ARGS_PASSTHRU);
@@ -2135,22 +2012,68 @@ PHP_GSHUTDOWN_FUNCTION(mongodb)
 PHP_MINFO_FUNCTION(mongodb)
 {
 	php_info_print_table_start();
-	php_info_print_table_header(2, "mongodb support", "enabled");
-	php_info_print_table_row(2, "mongodb version", MONGODB_VERSION_S);
-	php_info_print_table_row(2, "mongodb stability", MONGODB_STABILITY_S);
+	php_info_print_table_header(2, "MongoDB support", "enabled");
+	php_info_print_table_row(2, "MongoDB extension version", MONGODB_VERSION_S);
+	php_info_print_table_row(2, "MongoDB extension stability", MONGODB_STABILITY_S);
+
+#ifdef HAVE_SYSTEM_LIBBSON
+	php_info_print_table_row(2, "libbson headers version", BSON_VERSION_S);
+	php_info_print_table_row(2, "libbson library version", bson_get_version());
+#else
+	php_info_print_table_row(2, "libbson bundled version", BSON_VERSION_S);
+#endif
+
 #ifdef HAVE_SYSTEM_LIBMONGOC
 	php_info_print_table_row(2, "libmongoc headers version", MONGOC_VERSION_S);
 	php_info_print_table_row(2, "libmongoc library version", mongoc_get_version());
 #else
 	/* Bundled libraries, buildtime = runtime */
-	php_info_print_table_row(2, "libmongoc version", MONGOC_VERSION_S);
+	php_info_print_table_row(2, "libmongoc bundled version", MONGOC_VERSION_S);
 #endif
-#ifdef HAVE_SYSTEM_LIBBSON
-	php_info_print_table_row(2, "libbson headers version", BSON_VERSION_S);
-	php_info_print_table_row(2, "libbson library version", bson_get_version());
+
+#ifdef MONGOC_ENABLE_SSL
+	php_info_print_table_row(2, "libmongoc SSL", "enabled");
+# if defined(MONGOC_ENABLE_SSL_OPENSSL)
+	php_info_print_table_row(2, "libmongoc SSL library", "OpenSSL");
+# elif defined(MONGOC_ENABLE_SSL_LIBRESSL)
+	php_info_print_table_row(2, "libmongoc SSL library", "LibreSSL");
+# elif defined(MONGOC_ENABLE_SSL_SECURE_TRANSPORT)
+	php_info_print_table_row(2, "libmongoc SSL library", "Secure Transport");
+# elif defined(MONGOC_ENABLE_SSL_SECURE_CHANNEL)
+	php_info_print_table_row(2, "libmongoc SSL library", "Secure Channel");
+# else
+	php_info_print_table_row(2, "libmongoc SSL library", "unknown");
+# endif
 #else
-	php_info_print_table_row(2, "libbson version", BSON_VERSION_S);
+	php_info_print_table_row(2, "libmongoc SSL", "disabled");
 #endif
+
+#ifdef MONGOC_ENABLE_CRYPTO
+	php_info_print_table_row(2, "libmongoc crypto", "enabled");
+# if defined(MONGOC_ENABLE_CRYPTO_LIBCRYPTO)
+	php_info_print_table_row(2, "libmongoc crypto library", "libcrypto");
+# elif defined(MONGOC_ENABLE_CRYPTO_COMMON_CRYPTO)
+	php_info_print_table_row(2, "libmongoc crypto library", "Common Crypto");
+# elif defined(MONGOC_ENABLE_CRYPTO_CNG)
+	php_info_print_table_row(2, "libmongoc crypto library", "CNG");
+# else
+	php_info_print_table_row(2, "libmongoc crypto library", "unknown");
+# endif
+# ifdef MONGOC_ENABLE_CRYPTO_SYSTEM_PROFILE
+	php_info_print_table_row(2, "libmongoc crypto system profile", "enabled");
+# else
+	php_info_print_table_row(2, "libmongoc crypto system profile", "disabled");
+# endif
+#else
+	php_info_print_table_row(2, "libmongoc crypto", "disabled");
+#endif
+
+#ifdef MONGOC_ENABLE_SASL
+	php_info_print_table_row(2, "libmongoc SASL", "enabled");
+#else
+	php_info_print_table_row(2, "libmongoc SASL", "disabled");
+#endif
+
 	php_info_print_table_end();
 
 	DISPLAY_INI_ENTRIES();
