@@ -1,20 +1,18 @@
 /*
-  +----------------------------------------------------------------------+
-  | PHP Version 5                                                        |
-  +----------------------------------------------------------------------+
-  | Copyright (c) 2014-2015 Hannes Magnusson                             |
-  +----------------------------------------------------------------------+
-  | This source file is subject to version 3.01 of the PHP license,      |
-  | that is bundled with this package in the file LICENSE, and is        |
-  | available through the world-wide-web at the following url:           |
-  | http://www.php.net/license/3_01.txt                                  |
-  | If you did not receive a copy of the PHP license and are unable to   |
-  | obtain it through the world-wide-web, please send a note to          |
-  | license@php.net so we can mail you a copy immediately.               |
-  +----------------------------------------------------------------------+
-  | Author: Hannes Magnusson <bjori@php.net>                             |
-  +----------------------------------------------------------------------+
-*/
+ * Copyright 2014-2017 MongoDB, Inc.
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *   http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 #ifndef PHONGO_H
 #define PHONGO_H
@@ -27,7 +25,7 @@
 extern zend_module_entry mongodb_module_entry;
 
 /* FIXME: Its annoying to bump version. Move into phongo_version.h.in */
-#define MONGODB_VERSION_S "1.2.0"
+#define MONGODB_VERSION_S "1.2.3"
 #define MONGODB_STABILITY_S "stable"
 #define MONGODB_VERSION   MONGODB_VERSION_S
 
@@ -48,15 +46,18 @@ ZEND_BEGIN_MODULE_GLOBALS(mongodb)
 ZEND_END_MODULE_GLOBALS(mongodb)
 
 #if PHP_VERSION_ID >= 70000
-#   define MONGODB_G(v) ZEND_MODULE_GLOBALS_ACCESSOR(mongodb, v)
+# define MONGODB_G(v) ZEND_MODULE_GLOBALS_ACCESSOR(mongodb, v)
+# if defined(ZTS) && defined(COMPILE_DL_MONGODB)
+ZEND_TSRMLS_CACHE_EXTERN()
+# endif
 #else
-#   ifdef ZTS
-#   	define MONGODB_G(v) TSRMG(mongodb_globals_id, zend_mongodb_globals *, v)
-#   	define mglo mongodb_globals_id
-#   else
-#   	define MONGODB_G(v) (mongodb_globals.v)
-#   	define mglo mongodb_globals
-#   endif
+# ifdef ZTS
+#  define MONGODB_G(v) TSRMG(mongodb_globals_id, zend_mongodb_globals *, v)
+#  define mglo mongodb_globals_id
+# else
+#  define MONGODB_G(v) (mongodb_globals.v)
+#  define mglo mongodb_globals
+# endif
 #endif
 
 #define PHONGO_WRITE_CONCERN_W_MAJORITY "majority"
@@ -151,9 +152,6 @@ void php_phongo_new_regex_from_regex_and_options(zval *object, const char *patte
 zend_bool phongo_writeerror_init(zval *return_value, bson_t *bson TSRMLS_DC);
 zend_bool phongo_writeconcernerror_init(zval *return_value, bson_t *bson TSRMLS_DC);
 
-void php_phongo_cursor_free(php_phongo_cursor_t *cursor);
-zend_object_iterator* php_phongo_cursor_get_iterator(zend_class_entry *ce, zval *object, int by_ref TSRMLS_DC);
-
 #if PHP_VERSION_ID >= 70000
 #define PHONGO_CE_FINAL(ce) do {    \
 	ce->ce_flags |= ZEND_ACC_FINAL; \
