@@ -1,24 +1,18 @@
 /*
-  +---------------------------------------------------------------------------+
-  | PHP Driver for MongoDB                                                    |
-  +---------------------------------------------------------------------------+
-  | Copyright 2013-2015 MongoDB, Inc.                                         |
-  |                                                                           |
-  | Licensed under the Apache License, Version 2.0 (the "License");           |
-  | you may not use this file except in compliance with the License.          |
-  | You may obtain a copy of the License at                                   |
-  |                                                                           |
-  | http://www.apache.org/licenses/LICENSE-2.0                                |
-  |                                                                           |
-  | Unless required by applicable law or agreed to in writing, software       |
-  | distributed under the License is distributed on an "AS IS" BASIS,         |
-  | WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  |
-  | See the License for the specific language governing permissions and       |
-  | limitations under the License.                                            |
-  +---------------------------------------------------------------------------+
-  | Copyright (c) 2014-2015 MongoDB, Inc.                                     |
-  +---------------------------------------------------------------------------+
-*/
+ * Copyright 2014-2017 MongoDB, Inc.
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *   http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 #ifdef HAVE_CONFIG_H
 #	include "config.h"
