@@ -1,24 +1,18 @@
 /*
-  +---------------------------------------------------------------------------+
-  | PHP Driver for MongoDB                                                    |
-  +---------------------------------------------------------------------------+
-  | Copyright 2013-2015 MongoDB, Inc.                                         |
-  |                                                                           |
-  | Licensed under the Apache License, Version 2.0 (the "License");           |
-  | you may not use this file except in compliance with the License.          |
-  | You may obtain a copy of the License at                                   |
-  |                                                                           |
-  | http://www.apache.org/licenses/LICENSE-2.0                                |
-  |                                                                           |
-  | Unless required by applicable law or agreed to in writing, software       |
-  | distributed under the License is distributed on an "AS IS" BASIS,         |
-  | WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  |
-  | See the License for the specific language governing permissions and       |
-  | limitations under the License.                                            |
-  +---------------------------------------------------------------------------+
-  | Copyright (c) 2014-2015 MongoDB, Inc.                                     |
-  +---------------------------------------------------------------------------+
-*/
+ * Copyright 2014-2017 MongoDB, Inc.
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *   http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 #ifdef HAVE_CONFIG_H
 #	include "config.h"
@@ -46,6 +40,171 @@ PHONGO_API zend_class_entry *php_phongo_cursor_ce;
 
 zend_object_handlers php_phongo_handler_cursor;
 
+static void php_phongo_cursor_free_current(php_phongo_cursor_t *cursor) /* {{{ */
+{
+	if (!Z_ISUNDEF(cursor->visitor_data.zchild)) {
+		zval_ptr_dtor(&cursor->visitor_data.zchild);
+#if PHP_VERSION_ID >= 70000
+		ZVAL_UNDEF(&cursor->visitor_data.zchild);
+#else
+		cursor->visitor_data.zchild = NULL;
+#endif
+	}
+} /* }}} */
+
+/* {{{ Iterator handlers */
+static void php_phongo_cursor_iterator_dtor(zend_object_iterator *iter TSRMLS_DC) /* {{{ */
+{
+	php_phongo_cursor_iterator *cursor_it = (php_phongo_cursor_iterator *)iter;
+
+	if (!Z_ISUNDEF(cursor_it->intern.data)) {
+#if PHP_VERSION_ID >= 70000
+		zval_ptr_dtor(&cursor_it->intern.data);
+#else
+		zval_ptr_dtor((zval**)&cursor_it->intern.data);
+		cursor_it->intern.data = NULL;
+#endif
+	}
+
+#if PHP_VERSION_ID < 70000
+	efree(cursor_it);
+#endif
+} /* }}} */
+
+static int php_phongo_cursor_iterator_valid(zend_object_iterator *iter TSRMLS_DC) /* {{{ */
+{
+	php_phongo_cursor_t *cursor = ((php_phongo_cursor_iterator *)iter)->cursor;
+
+	if (!Z_ISUNDEF(cursor->visitor_data.zchild)) {
+		return SUCCESS;
+	}
+
+	return FAILURE;
+} /* }}} */
+
+#if PHP_VERSION_ID < 50500
+static int php_phongo_cursor_iterator_get_current_key(zend_object_iterator *iter, char **str_key, uint *str_key_len, ulong *int_key TSRMLS_DC) /* {{{ */
+{
+	php_phongo_cursor_t *cursor = ((php_phongo_cursor_iterator *)iter)->cursor;
+
+	*int_key = (ulong) cursor->current;
+	return HASH_KEY_IS_LONG;
+} /* }}} */
+#else
+static void php_phongo_cursor_iterator_get_current_key(zend_object_iterator *iter, zval *key TSRMLS_DC) /* {{{ */
+{
+	php_phongo_cursor_t *cursor = ((php_phongo_cursor_iterator *)iter)->cursor;
+
+	ZVAL_LONG(key, cursor->current);
+} /* }}} */
+#endif
+
+#if PHP_VERSION_ID < 70000
+static void php_phongo_cursor_iterator_get_current_data(zend_object_iterator *iter, zval ***data TSRMLS_DC) /* {{{ */
+{
+	php_phongo_cursor_t *cursor = ((php_phongo_cursor_iterator *)iter)->cursor;
+
+	*data = &cursor->visitor_data.zchild;
+} /* }}} */
+#else
+static zval* php_phongo_cursor_iterator_get_current_data(zend_object_iterator *iter) /* {{{ */
+{
+	php_phongo_cursor_t *cursor = ((php_phongo_cursor_iterator *)iter)->cursor;
+
+	return &cursor->visitor_data.zchild;
+} /* }}} */
+#endif
+
+static void php_phongo_cursor_iterator_move_forward(zend_object_iterator *iter TSRMLS_DC) /* {{{ */
+{
+	php_phongo_cursor_iterator *cursor_it = (php_phongo_cursor_iterator *)iter;
+	php_phongo_cursor_t        *cursor = cursor_it->cursor;
+	const bson_t               *doc;
+
+	php_phongo_cursor_free_current(cursor);
+	cursor->current++;
+
+	if (mongoc_cursor_next(cursor->cursor, &doc)) {
+		phongo_bson_to_zval_ex(bson_get_data(doc), doc->len, &cursor->visitor_data);
+	} else {
+		bson_error_t error;
+
+		if (mongoc_cursor_error(cursor->cursor, &error)) {
+			/* Intentionally not destroying the cursor as it will happen
+			 * naturally now that there are no more results */
+			phongo_throw_exception_from_bson_error_t(&error TSRMLS_CC);
+		}
+	}
+} /* }}} */
+
+static void php_phongo_cursor_iterator_rewind(zend_object_iterator *iter TSRMLS_DC) /* {{{ */
+{
+	php_phongo_cursor_iterator *cursor_it = (php_phongo_cursor_iterator *)iter;
+	php_phongo_cursor_t        *cursor = cursor_it->cursor;
+	const bson_t               *doc;
+
+	if (cursor->current > 0) {
+		phongo_throw_exception(PHONGO_ERROR_LOGIC TSRMLS_CC, "Cursors cannot rewind after starting iteration");
+		return;
+	}
+
+	php_phongo_cursor_free_current(cursor);
+
+	doc = mongoc_cursor_current(cursor->cursor);
+
+	if (doc) {
+		phongo_bson_to_zval_ex(bson_get_data(doc), doc->len, &cursor->visitor_data);
+	}
+} /* }}} */
+
+/* iterator handler table */
+zend_object_iterator_funcs php_phongo_cursor_iterator_funcs = {
+	php_phongo_cursor_iterator_dtor,
+	php_phongo_cursor_iterator_valid,
+	php_phongo_cursor_iterator_get_current_data,
+	php_phongo_cursor_iterator_get_current_key,
+	php_phongo_cursor_iterator_move_forward,
+	php_phongo_cursor_iterator_rewind,
+	NULL /* invalidate_current is not used */
+};
+
+zend_object_iterator *php_phongo_cursor_get_iterator(zend_class_entry *ce, zval *object, int by_ref TSRMLS_DC) /* {{{ */
+{
+	php_phongo_cursor_iterator *cursor_it = NULL;
+	php_phongo_cursor_t *cursor = Z_CURSOR_OBJ_P(object);
+
+	if (by_ref) {
+		zend_error(E_ERROR, "An iterator cannot be used with foreach by reference");
+	}
+
+	if (cursor->got_iterator) {
+		phongo_throw_exception(PHONGO_ERROR_LOGIC TSRMLS_CC, "Cursors cannot yield multiple iterators");
+		return NULL;
+	}
+
+	cursor->got_iterator = 1;
+
+	cursor_it = ecalloc(1, sizeof(php_phongo_cursor_iterator));
+#if PHP_VERSION_ID >= 70000
+	zend_iterator_init(&cursor_it->intern);
+#endif
+
+#if PHP_VERSION_ID >= 70000
+	ZVAL_COPY(&cursor_it->intern.data, object);
+#else
+	Z_ADDREF_P(object);
+	cursor_it->intern.data  = (void*)object;
+#endif
+	cursor_it->intern.funcs = &php_phongo_cursor_iterator_funcs;
+	cursor_it->cursor = cursor;
+	/* cursor_it->current should already be allocated to zero */
+
+	php_phongo_cursor_free_current(cursor_it->cursor);
+
+	return &cursor_it->intern;
+} /* }}} */
+ /* }}} */
+
 /* {{{ proto void Cursor::setTypeMap(array $typemap)
    Sets a type map to use for BSON unserialization */
 PHP_METHOD(Cursor, setTypeMap)
@@ -62,9 +221,23 @@ PHP_METHOD(Cursor, setTypeMap)
 		return;
 	}
 
+	/* Check if the existing element needs to be freed before we overwrite
+	 * visitor_data, which contains the only reference to it. */
+	if (!Z_ISUNDEF(intern->visitor_data.zchild)) {
+		php_phongo_cursor_free_current(intern);
+	}
+
 	phongo_bson_typemap_to_state(typemap, &state.map TSRMLS_CC);
 
 	intern->visitor_data = state;
+
+	/* If the cursor has a current element, we just freed it and should restore
+	 * it with a new type map applied. */
+	if (mongoc_cursor_current(intern->cursor)) {
+		const bson_t *doc = mongoc_cursor_current(intern->cursor);
+
+		phongo_bson_to_zval_ex(bson_get_data(doc), doc->len, &intern->visitor_data);
+	}
 }
 /* }}} */
 
@@ -211,6 +384,10 @@ static void php_phongo_cursor_free_object(phongo_free_object_arg *object TSRMLS_
 
 	zend_object_std_dtor(&intern->std TSRMLS_CC);
 
+	if (intern->cursor) {
+		mongoc_cursor_destroy(intern->cursor);
+	}
+
 	if (intern->database) {
 		efree(intern->database);
 	}
@@ -231,7 +408,7 @@ static void php_phongo_cursor_free_object(phongo_free_object_arg *object TSRMLS_
 		zval_ptr_dtor(&intern->read_preference);
 	}
 
-	php_phongo_cursor_free(intern);
+	php_phongo_cursor_free_current(intern);
 
 	zval_ptr_dtor(&intern->manager);
 
