@@ -1,24 +1,18 @@
 /*
-  +---------------------------------------------------------------------------+
-  | PHP Driver for MongoDB                                                    |
-  +---------------------------------------------------------------------------+
-  | Copyright 2013-2015 MongoDB, Inc.                                         |
-  |                                                                           |
-  | Licensed under the Apache License, Version 2.0 (the "License");           |
-  | you may not use this file except in compliance with the License.          |
-  | You may obtain a copy of the License at                                   |
-  |                                                                           |
-  | http://www.apache.org/licenses/LICENSE-2.0                                |
-  |                                                                           |
-  | Unless required by applicable law or agreed to in writing, software       |
-  | distributed under the License is distributed on an "AS IS" BASIS,         |
-  | WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  |
-  | See the License for the specific language governing permissions and       |
-  | limitations under the License.                                            |
-  +---------------------------------------------------------------------------+
-  | Copyright (c) 2014-2015 MongoDB, Inc.                                     |
-  +---------------------------------------------------------------------------+
-*/
+ * Copyright 2014-2017 MongoDB, Inc.
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *   http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 #ifdef HAVE_CONFIG_H
 #	include "config.h"
@@ -88,7 +82,7 @@ static bool php_phongo_manager_merge_context_options(zval *zdriverOptions TSRMLS
 
 	/* Perform array union (see: add_function() in zend_operators.c) */
 #if PHP_VERSION_ID >= 70000
-	zend_hash_merge(Z_ARRVAL_P(zdriverOptions), Z_ARRVAL_P(zcontextOptions), zval_add_ref, 0);	
+	zend_hash_merge(Z_ARRVAL_P(zdriverOptions), Z_ARRVAL_P(zcontextOptions), zval_add_ref, 0);
 #else
 	{
 		zval *tmp;
@@ -129,6 +123,8 @@ static void php_phongo_manager_prep_tagsets(zval *options TSRMLS_DC)
 			/* php_phongo_make_uri() and php_phongo_apply_rp_options_to_uri()
 			 * are both case-insensitive, so we need to be as well. */
 			if (!strcasecmp(ZSTR_VAL(string_key), "readpreferencetags")) {
+				ZVAL_DEREF(tagSets);
+				SEPARATE_ZVAL_NOREF(tagSets);
 				php_phongo_read_preference_prep_tagsets(tagSets TSRMLS_CC);
 			}
 		} ZEND_HASH_FOREACH_END();
@@ -152,6 +148,7 @@ static void php_phongo_manager_prep_tagsets(zval *options TSRMLS_DC)
 			/* php_phongo_make_uri() and php_phongo_apply_rp_options_to_uri()
 			 * are both case-insensitive, so we need to be as well. */
 			if (!strcasecmp(string_key, "readpreferencetags")) {
+				SEPARATE_ZVAL_IF_NOT_REF(tagSets);
 				php_phongo_read_preference_prep_tagsets(*tagSets TSRMLS_CC);
 			}
 		}
@@ -177,9 +174,10 @@ PHP_METHOD(Manager, __construct)
 	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling TSRMLS_CC);
 	intern = Z_MANAGER_OBJ_P(getThis());
 
-	/* Separate the driverOptions zval, since we may end up modifying it in
-	 * php_phongo_manager_merge_context_options() below. */
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|s!a!a!", &uri_string, &uri_string_len, &options, &driverOptions) == FAILURE) {
+	/* Separate the options and driverOptions zvals, since we may end up
+	 * modifying them in php_phongo_manager_prep_tagsets() and
+	 * php_phongo_manager_merge_context_options() below, respectively. */
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|s!a/!a/!", &uri_string, &uri_string_len, &options, &driverOptions) == FAILURE) {
 		zend_restore_error_handling(&error_handling TSRMLS_CC);
 		return;
 	}
