@@ -1,22 +1,18 @@
 /*
-  +----------------------------------------------------------------------+
-  | PHP Version 5                                                        |
-  +----------------------------------------------------------------------+
-  | Copyright (c) 2014-2015 Hannes Magnusson                             |
-  +----------------------------------------------------------------------+
-  | This source file is subject to version 3.01 of the PHP license,      |
-  | that is bundled with this package in the file LICENSE, and is        |
-  | available through the world-wide-web at the following url:           |
-  | http://www.php.net/license/3_01.txt                                  |
-  | If you did not receive a copy of the PHP license and are unable to   |
-  | obtain it through the world-wide-web, please send a note to          |
-  | license@php.net so we can mail you a copy immediately.               |
-  +----------------------------------------------------------------------+
-  | Author: Hannes Magnusson <bjori@php.net>                             |
-  +----------------------------------------------------------------------+
-*/
-
-/* $Id$ */
+ * Copyright 2014-2017 MongoDB, Inc.
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *   http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 #ifdef HAVE_CONFIG_H
 #	include "config.h"
@@ -208,7 +204,11 @@ bool php_phongo_bson_visit_double(const bson_iter_t *iter ARG_UNUSED, const char
 	zval *retval = ((php_phongo_bson_state *)data)->zchild;
 #endif
 
-	add_assoc_double(retval, key, v_double);
+	if (((php_phongo_bson_state *)data)->is_visiting_array) {
+		add_next_index_double(retval, v_double);
+	} else {
+		add_assoc_double(retval, key, v_double);
+	}
 
 	return false;
 }
@@ -221,7 +221,11 @@ bool php_phongo_bson_visit_utf8(const bson_iter_t *iter ARG_UNUSED, const char *
 	zval *retval = ((php_phongo_bson_state *)data)->zchild;
 #endif
 
-	ADD_ASSOC_STRING_EX(retval, key, strlen(key), v_utf8, v_utf8_len);
+	if (((php_phongo_bson_state *)data)->is_visiting_array) {
+		ADD_NEXT_INDEX_STRINGL(retval, v_utf8, v_utf8_len);
+	} else {
+		ADD_ASSOC_STRING_EX(retval, key, strlen(key), v_utf8, v_utf8_len);
+	}
 
 	return false;
 }
@@ -255,7 +259,11 @@ bool php_phongo_bson_visit_binary(const bson_iter_t *iter ARG_UNUSED, const char
 
 		php_phongo_new_binary_from_binary_and_type(&zchild, (const char *)v_binary, v_binary_len, v_subtype TSRMLS_CC);
 
-		ADD_ASSOC_ZVAL(retval, key, &zchild);
+		if (((php_phongo_bson_state *)data)->is_visiting_array) {
+			add_next_index_zval(retval, &zchild);
+		} else {
+			ADD_ASSOC_ZVAL(retval, key, &zchild);
+		}
 		Z_SET_REFCOUNT(zchild, 1);
 #else
 		zval *zchild = NULL;
@@ -263,7 +271,11 @@ bool php_phongo_bson_visit_binary(const bson_iter_t *iter ARG_UNUSED, const char
 		MAKE_STD_ZVAL(zchild);
 		php_phongo_new_binary_from_binary_and_type(zchild, (const char *)v_binary, v_binary_len, v_subtype TSRMLS_CC);
 
-		ADD_ASSOC_ZVAL(retval, key, zchild);
+		if (((php_phongo_bson_state *)data)->is_visiting_array) {
+			add_next_index_zval(retval, zchild);
+		} else {
+			ADD_ASSOC_ZVAL(retval, key, zchild);
+		}
 		Z_SET_REFCOUNT_P(zchild, 1);
 #endif
 	}
@@ -292,7 +304,12 @@ bool php_phongo_bson_visit_oid(const bson_iter_t *iter ARG_UNUSED, const char *k
 	zval zchild;
 
 	php_phongo_objectid_new_from_oid(&zchild, v_oid TSRMLS_CC);
-	ADD_ASSOC_ZVAL(retval, key, &zchild);
+
+	if (((php_phongo_bson_state *)data)->is_visiting_array) {
+		add_next_index_zval(retval, &zchild);
+	} else {
+		ADD_ASSOC_ZVAL(retval, key, &zchild);
+	}
 #else
 	zval *zchild = NULL;
 	TSRMLS_FETCH();
@@ -300,7 +317,11 @@ bool php_phongo_bson_visit_oid(const bson_iter_t *iter ARG_UNUSED, const char *k
 	MAKE_STD_ZVAL(zchild);
 	php_phongo_objectid_new_from_oid(zchild, v_oid TSRMLS_CC);
 
-	ADD_ASSOC_ZVAL(retval, key, zchild);
+	if (((php_phongo_bson_state *)data)->is_visiting_array) {
+		add_next_index_zval(retval, zchild);
+	} else {
+		ADD_ASSOC_ZVAL(retval, key, zchild);
+	}
 	Z_SET_REFCOUNT_P(zchild, 1);
 #endif
 
@@ -315,7 +336,11 @@ bool php_phongo_bson_visit_bool(const bson_iter_t *iter ARG_UNUSED, const char *
 	zval *retval = ((php_phongo_bson_state *)data)->zchild;
 #endif
 
-	add_assoc_bool(retval, key, v_bool);
+	if (((php_phongo_bson_state *)data)->is_visiting_array) {
+		add_next_index_bool(retval, v_bool);
+	} else {
+		add_assoc_bool(retval, key, v_bool);
+	}
 
 	return false;
 }
@@ -331,7 +356,12 @@ bool php_phongo_bson_visit_date_time(const bson_iter_t *iter ARG_UNUSED, const c
 	zval zchild;
 
 	php_phongo_new_utcdatetime_from_epoch(&zchild, msec_since_epoch TSRMLS_CC);
-	ADD_ASSOC_ZVAL(retval, key, &zchild);
+
+	if (((php_phongo_bson_state *)data)->is_visiting_array) {
+		add_next_index_zval(retval, &zchild);
+	} else {
+		ADD_ASSOC_ZVAL(retval, key, &zchild);
+	}
 #else
 	zval *zchild = NULL;
 	TSRMLS_FETCH();
@@ -339,7 +369,11 @@ bool php_phongo_bson_visit_date_time(const bson_iter_t *iter ARG_UNUSED, const c
 	MAKE_STD_ZVAL(zchild);
 	php_phongo_new_utcdatetime_from_epoch(zchild, msec_since_epoch TSRMLS_CC);
 
-	ADD_ASSOC_ZVAL(retval, key, zchild);
+	if (((php_phongo_bson_state *)data)->is_visiting_array) {
+		add_next_index_zval(retval, zchild);
+	} else {
+		ADD_ASSOC_ZVAL(retval, key, zchild);
+	}
 	Z_SET_REFCOUNT_P(zchild, 1);
 #endif
 
@@ -354,7 +388,12 @@ bool php_phongo_bson_visit_decimal128(const bson_iter_t *iter ARG_UNUSED, const
 	zval zchild;
 
 	php_phongo_new_decimal128(&zchild, decimal TSRMLS_CC);
-	ADD_ASSOC_ZVAL(retval, key, &zchild);
+
+	if (((php_phongo_bson_state *)data)->is_visiting_array) {
+		add_next_index_zval(retval, &zchild);
+	} else {
+		ADD_ASSOC_ZVAL(retval, key, &zchild);
+	}
 #else
 	zval *retval = ((php_phongo_bson_state *)data)->zchild;
 	zval *zchild = NULL;
@@ -363,7 +402,11 @@ bool php_phongo_bson_visit_decimal128(const bson_iter_t *iter ARG_UNUSED, const
 	MAKE_STD_ZVAL(zchild);
 	php_phongo_new_decimal128(zchild, decimal TSRMLS_CC);
 
-	ADD_ASSOC_ZVAL(retval, key, zchild);
+	if (((php_phongo_bson_state *)data)->is_visiting_array) {
+		add_next_index_zval(retval, zchild);
+	} else {
+		ADD_ASSOC_ZVAL(retval, key, zchild);
+	}
 	Z_SET_REFCOUNT_P(zchild, 1);
 #endif
 
@@ -379,7 +422,11 @@ bool php_phongo_bson_visit_null(const bson_iter_t *iter ARG_UNUSED, const char *
 	zval *retval = ((php_phongo_bson_state *)data)->zchild;
 #endif
 
-	add_assoc_null(retval, key);
+	if (((php_phongo_bson_state *)data)->is_visiting_array) {
+		add_next_index_null(retval);
+	} else {
+		add_assoc_null(retval, key);
+	}
 
 	return false;
 }
@@ -396,7 +443,11 @@ bool php_phongo_bson_visit_regex(const bson_iter_t *iter ARG_UNUSED, const char
 
 	php_phongo_new_regex_from_regex_and_options(&zchild, v_regex, v_options TSRMLS_CC);
 
-	ADD_ASSOC_ZVAL(retval, key, &zchild);
+	if (((php_phongo_bson_state *)data)->is_visiting_array) {
+		add_next_index_zval(retval, &zchild);
+	} else {
+		ADD_ASSOC_ZVAL(retval, key, &zchild);
+	}
 #else
 	zval *zchild = NULL;
 	TSRMLS_FETCH();
@@ -404,7 +455,11 @@ bool php_phongo_bson_visit_regex(const bson_iter_t *iter ARG_UNUSED, const char
 	MAKE_STD_ZVAL(zchild);
 	php_phongo_new_regex_from_regex_and_options(zchild, v_regex, v_options TSRMLS_CC);
 
-	ADD_ASSOC_ZVAL(retval, key, zchild);
+	if (((php_phongo_bson_state *)data)->is_visiting_array) {
+		add_next_index_zval(retval, zchild);
+	} else {
+		ADD_ASSOC_ZVAL(retval, key, zchild);
+	}
 	Z_SET_REFCOUNT_P(zchild, 1);
 #endif
 
@@ -432,7 +487,11 @@ bool php_phongo_bson_visit_code(const bson_iter_t *iter ARG_UNUSED, const char *
 
 	php_phongo_new_javascript_from_javascript(1, &zchild, v_code, v_code_len TSRMLS_CC);
 
-	ADD_ASSOC_ZVAL(retval, key, &zchild);
+	if (((php_phongo_bson_state *)data)->is_visiting_array) {
+		add_next_index_zval(retval, &zchild);
+	} else {
+		ADD_ASSOC_ZVAL(retval, key, &zchild);
+	}
 #else
 	zval *zchild = NULL;
 	TSRMLS_FETCH();
@@ -440,7 +499,11 @@ bool php_phongo_bson_visit_code(const bson_iter_t *iter ARG_UNUSED, const char *
 	MAKE_STD_ZVAL(zchild);
 	php_phongo_new_javascript_from_javascript(1, zchild, v_code, v_code_len TSRMLS_CC);
 
-	ADD_ASSOC_ZVAL(retval, key, zchild);
+	if (((php_phongo_bson_state *)data)->is_visiting_array) {
+		add_next_index_zval(retval, zchild);
+	} else {
+		ADD_ASSOC_ZVAL(retval, key, zchild);
+	}
 	Z_SET_REFCOUNT_P(zchild, 1);
 #endif
 
@@ -467,8 +530,12 @@ bool php_phongo_bson_visit_codewscope(const bson_iter_t *iter ARG_UNUSED, const
 	zval zchild;
 
 	php_phongo_new_javascript_from_javascript_and_scope(1, &zchild, v_code, v_code_len, v_scope TSRMLS_CC);
-	ADD_ASSOC_ZVAL(retval, key, &zchild);
 
+	if (((php_phongo_bson_state *)data)->is_visiting_array) {
+		add_next_index_zval(retval, &zchild);
+	} else {
+		ADD_ASSOC_ZVAL(retval, key, &zchild);
+	}
 #else
 	zval *zchild = NULL;
 	TSRMLS_FETCH();
@@ -476,7 +543,11 @@ bool php_phongo_bson_visit_codewscope(const bson_iter_t *iter ARG_UNUSED, const
 	MAKE_STD_ZVAL(zchild);
 	php_phongo_new_javascript_from_javascript_and_scope(1, zchild, v_code, v_code_len, v_scope TSRMLS_CC);
 
-	ADD_ASSOC_ZVAL(retval, key, zchild);
+	if (((php_phongo_bson_state *)data)->is_visiting_array) {
+		add_next_index_zval(retval, zchild);
+	} else {
+		ADD_ASSOC_ZVAL(retval, key, zchild);
+	}
 	Z_SET_REFCOUNT_P(zchild, 1);
 #endif
 
@@ -491,7 +562,11 @@ bool php_phongo_bson_visit_int32(const bson_iter_t *iter ARG_UNUSED, const char
 	zval *retval = ((php_phongo_bson_state *)data)->zchild;
 #endif
 
-	add_assoc_long(retval, key, v_int32);
+	if (((php_phongo_bson_state *)data)->is_visiting_array) {
+		add_next_index_long(retval, v_int32);
+	} else {
+		add_assoc_long(retval, key, v_int32);
+	}
 
 	return false;
 }
@@ -508,7 +583,11 @@ bool php_phongo_bson_visit_timestamp(const bson_iter_t *iter ARG_UNUSED, const c
 
 	php_phongo_new_timestamp_from_increment_and_timestamp(&zchild, v_increment, v_timestamp TSRMLS_CC);
 
-	ADD_ASSOC_ZVAL(retval, key, &zchild);
+	if (((php_phongo_bson_state *)data)->is_visiting_array) {
+		add_next_index_zval(retval, &zchild);
+	} else {
+		ADD_ASSOC_ZVAL(retval, key, &zchild);
+	}
 #else
 	zval *zchild = NULL;
 	TSRMLS_FETCH();
@@ -516,7 +595,11 @@ bool php_phongo_bson_visit_timestamp(const bson_iter_t *iter ARG_UNUSED, const c
 	MAKE_STD_ZVAL(zchild);
 	php_phongo_new_timestamp_from_increment_and_timestamp(zchild, v_increment, v_timestamp TSRMLS_CC);
 
-	ADD_ASSOC_ZVAL(retval, key, zchild);
+	if (((php_phongo_bson_state *)data)->is_visiting_array) {
+		add_next_index_zval(retval, zchild);
+	} else {
+		ADD_ASSOC_ZVAL(retval, key, zchild);
+	}
 	Z_SET_REFCOUNT_P(zchild, 1);
 #endif
 
@@ -530,9 +613,15 @@ bool php_phongo_bson_visit_int64(const bson_iter_t *iter ARG_UNUSED, const char
 #else
 	zval *retval = ((php_phongo_bson_state *)data)->zchild;
 #endif
+#if SIZEOF_PHONGO_LONG == 4
 	TSRMLS_FETCH();
+#endif
 
-	ADD_ASSOC_INT64(retval, key, v_int64);
+	if (((php_phongo_bson_state *)data)->is_visiting_array) {
+		ADD_NEXT_INDEX_INT64(retval, v_int64);
+	} else {
+		ADD_ASSOC_INT64(retval, key, v_int64);
+	}
 
 	return false;
 }
@@ -548,7 +637,12 @@ bool php_phongo_bson_visit_maxkey(const bson_iter_t *iter ARG_UNUSED, const char
 	zval zchild;
 
 	object_init_ex(&zchild, php_phongo_maxkey_ce);
-	ADD_ASSOC_ZVAL(retval, key, &zchild);
+
+	if (((php_phongo_bson_state *)data)->is_visiting_array) {
+		add_next_index_zval(retval, &zchild);
+	} else {
+		ADD_ASSOC_ZVAL(retval, key, &zchild);
+	}
 #else
 	zval *zchild = NULL;
 	TSRMLS_FETCH();
@@ -556,7 +650,11 @@ bool php_phongo_bson_visit_maxkey(const bson_iter_t *iter ARG_UNUSED, const char
 	MAKE_STD_ZVAL(zchild);
 	object_init_ex(zchild, php_phongo_maxkey_ce);
 
-	ADD_ASSOC_ZVAL(retval, key, zchild);
+	if (((php_phongo_bson_state *)data)->is_visiting_array) {
+		add_next_index_zval(retval, zchild);
+	} else {
+		ADD_ASSOC_ZVAL(retval, key, zchild);
+	}
 	Z_SET_REFCOUNT_P(zchild, 1);
 #endif
 
@@ -574,7 +672,12 @@ bool php_phongo_bson_visit_minkey(const bson_iter_t *iter ARG_UNUSED, const char
 	zval zchild;
 
 	object_init_ex(&zchild, php_phongo_minkey_ce);
-	ADD_ASSOC_ZVAL(retval, key, &zchild);
+
+	if (((php_phongo_bson_state *)data)->is_visiting_array) {
+		add_next_index_zval(retval, &zchild);
+	} else {
+		ADD_ASSOC_ZVAL(retval, key, &zchild);
+	}
 #else
 	zval *zchild = NULL;
 	TSRMLS_FETCH();
@@ -583,7 +686,11 @@ bool php_phongo_bson_visit_minkey(const bson_iter_t *iter ARG_UNUSED, const char
 	object_init_ex(zchild, php_phongo_minkey_ce);
 
 	Z_SET_REFCOUNT_P(zchild, 1);
-	ADD_ASSOC_ZVAL(retval, key, zchild);
+	if (((php_phongo_bson_state *)data)->is_visiting_array) {
+		add_next_index_zval(retval, zchild);
+	} else {
+		ADD_ASSOC_ZVAL(retval, key, zchild);
+	}
 #endif
 
 	return false;
@@ -653,10 +760,18 @@ bool php_phongo_bson_visit_document(const bson_iter_t *iter ARG_UNUSED, const ch
 			switch(state.map.document_type) {
 				case PHONGO_TYPEMAP_NATIVE_ARRAY:
 #if PHP_VERSION_ID >= 70000
-					ADD_ASSOC_ZVAL(retval, key, &state.zchild);
+					if (((php_phongo_bson_state *)data)->is_visiting_array) {
+						add_next_index_zval(retval, &state.zchild);
+					} else {
+						ADD_ASSOC_ZVAL(retval, key, &state.zchild);
+					}
 					Z_SET_REFCOUNT(state.zchild, 1);
 #else
-					ADD_ASSOC_ZVAL(retval, key, state.zchild);
+					if (((php_phongo_bson_state *)data)->is_visiting_array) {
+						add_next_index_zval(retval, state.zchild);
+					} else {
+						ADD_ASSOC_ZVAL(retval, key, state.zchild);
+					}
 					Z_SET_REFCOUNT_P(state.zchild, 1);
 #endif
 					break;
@@ -667,7 +782,11 @@ bool php_phongo_bson_visit_document(const bson_iter_t *iter ARG_UNUSED, const ch
 
 					object_init_ex(&obj, state.odm ? state.odm : state.map.document);
 					zend_call_method_with_1_params(&obj, NULL, NULL, BSON_UNSERIALIZE_FUNC_NAME, NULL, &state.zchild);
-					add_assoc_zval(retval, key, &obj);
+					if (((php_phongo_bson_state *)data)->is_visiting_array) {
+						add_next_index_zval(retval, &obj);
+					} else {
+						ADD_ASSOC_ZVAL(retval, key, &obj);
+					}
 					zval_ptr_dtor(&state.zchild);
 #else
 					zval *obj = NULL;
@@ -675,7 +794,11 @@ bool php_phongo_bson_visit_document(const bson_iter_t *iter ARG_UNUSED, const ch
 					MAKE_STD_ZVAL(obj);
 					object_init_ex(obj, state.odm ? state.odm : state.map.document);
 					zend_call_method_with_1_params(&obj, NULL, NULL, BSON_UNSERIALIZE_FUNC_NAME, NULL, state.zchild);
-					add_assoc_zval(retval, key, obj);
+					if (((php_phongo_bson_state *)data)->is_visiting_array) {
+						add_next_index_zval(retval, obj);
+					} else {
+						ADD_ASSOC_ZVAL(retval, key, obj);
+					}
 					zval_ptr_dtor(&state.zchild);
 #endif
 					break;
@@ -685,11 +808,19 @@ bool php_phongo_bson_visit_document(const bson_iter_t *iter ARG_UNUSED, const ch
 				default:
 #if PHP_VERSION_ID >= 70000
 					object_and_properties_init(&state.zchild, zend_standard_class_def, Z_ARRVAL(state.zchild));
-					add_assoc_zval(retval, key, &state.zchild);
+					if (((php_phongo_bson_state *)data)->is_visiting_array) {
+						add_next_index_zval(retval, &state.zchild);
+					} else {
+						ADD_ASSOC_ZVAL(retval, key, &state.zchild);
+					}
 					Z_SET_REFCOUNT(state.zchild, 1);
 #else
 					object_and_properties_init(state.zchild, zend_standard_class_def, Z_ARRVAL_P(state.zchild));
-					add_assoc_zval(retval, key, state.zchild);
+					if (((php_phongo_bson_state *)data)->is_visiting_array) {
+						add_next_index_zval(retval, state.zchild);
+					} else {
+						ADD_ASSOC_ZVAL(retval, key, state.zchild);
+					}
 					Z_SET_REFCOUNT_P(state.zchild, 1);
 #endif
 			}
@@ -721,6 +852,12 @@ bool php_phongo_bson_visit_array(const bson_iter_t *iter ARG_UNUSED, const char
 
 		state.map = ((php_phongo_bson_state *)data)->map;
 
+		/* Note that we are visiting an array, so element visitors know to use
+		 * add_next_index() (i.e. disregard BSON keys) instead of add_assoc()
+		 * when building the PHP array.
+		 */
+		state.is_visiting_array = true;
+
 #if PHP_VERSION_ID >= 70000
 		array_init(&state.zchild);
 #else
@@ -737,7 +874,11 @@ bool php_phongo_bson_visit_array(const bson_iter_t *iter ARG_UNUSED, const char
 
 					object_init_ex(&obj, state.map.array);
 					zend_call_method_with_1_params(&obj, NULL, NULL, BSON_UNSERIALIZE_FUNC_NAME, NULL, &state.zchild);
-					add_assoc_zval(retval, key, &obj);
+					if (((php_phongo_bson_state *)data)->is_visiting_array) {
+						add_next_index_zval(retval, &obj);
+					} else {
+						ADD_ASSOC_ZVAL(retval, key, &obj);
+					}
 					zval_ptr_dtor(&state.zchild);
 #else
 					zval *obj = NULL;
@@ -745,7 +886,11 @@ bool php_phongo_bson_visit_array(const bson_iter_t *iter ARG_UNUSED, const char
 					MAKE_STD_ZVAL(obj);
 					object_init_ex(obj, state.map.array);
 					zend_call_method_with_1_params(&obj, NULL, NULL, BSON_UNSERIALIZE_FUNC_NAME, NULL, state.zchild);
-					add_assoc_zval(retval, key, obj);
+					if (((php_phongo_bson_state *)data)->is_visiting_array) {
+						add_next_index_zval(retval, obj);
+					} else {
+						ADD_ASSOC_ZVAL(retval, key, obj);
+					}
 					zval_ptr_dtor(&state.zchild);
 #endif
 					break;
@@ -754,11 +899,19 @@ bool php_phongo_bson_visit_array(const bson_iter_t *iter ARG_UNUSED, const char
 				case PHONGO_TYPEMAP_NATIVE_OBJECT:
 #if PHP_VERSION_ID >= 70000
 					object_and_properties_init(&state.zchild, zend_standard_class_def, Z_ARRVAL(state.zchild));
-					add_assoc_zval(retval, key, &state.zchild);
+					if (((php_phongo_bson_state *)data)->is_visiting_array) {
+						add_next_index_zval(retval, &state.zchild);
+					} else {
+						ADD_ASSOC_ZVAL(retval, key, &state.zchild);
+					}
 					Z_SET_REFCOUNT(state.zchild, 1);
 #else
 					object_and_properties_init(state.zchild, zend_standard_class_def, Z_ARRVAL_P(state.zchild));
-					add_assoc_zval(retval, key, state.zchild);
+					if (((php_phongo_bson_state *)data)->is_visiting_array) {
+						add_next_index_zval(retval, state.zchild);
+					} else {
+						ADD_ASSOC_ZVAL(retval, key, state.zchild);
+					}
 					Z_SET_REFCOUNT_P(state.zchild, 1);
 #endif
 					break;
@@ -766,10 +919,18 @@ bool php_phongo_bson_visit_array(const bson_iter_t *iter ARG_UNUSED, const char
 				case PHONGO_TYPEMAP_NATIVE_ARRAY:
 				default:
 #if PHP_VERSION_ID >= 70000
-					ADD_ASSOC_ZVAL(retval, key, &state.zchild);
+					if (((php_phongo_bson_state *)data)->is_visiting_array) {
+						add_next_index_zval(retval, &state.zchild);
+					} else {
+						ADD_ASSOC_ZVAL(retval, key, &state.zchild);
+					}
 					Z_SET_REFCOUNT(state.zchild, 1);
 #else
-					ADD_ASSOC_ZVAL(retval, key, state.zchild);
+					if (((php_phongo_bson_state *)data)->is_visiting_array) {
+						add_next_index_zval(retval, state.zchild);
+					} else {
+						ADD_ASSOC_ZVAL(retval, key, state.zchild);
+					}
 					Z_SET_REFCOUNT_P(state.zchild, 1);
 #endif
 					break;
@@ -905,8 +1066,8 @@ void object_to_bson(zval *object, php_phongo_bson_flags_t flags, const char *key
 			tmp_ht = HASH_OF(obj_data);
 #endif
 
-			if (tmp_ht) {
-				ZEND_HASH_APPLY_COUNT(tmp_ht)++;
+			if (tmp_ht && ZEND_HASH_APPLY_PROTECTION(tmp_ht)) {
+				ZEND_HASH_INC_APPLY_COUNT(tmp_ht);
 			}
 
 			/* Persistable objects must always be serialized as BSON documents;
@@ -940,8 +1101,8 @@ void object_to_bson(zval *object, php_phongo_bson_flags_t flags, const char *key
 				bson_append_array_end(bson, &child);
 			}
 
-			if (tmp_ht) {
-				ZEND_HASH_APPLY_COUNT(tmp_ht)--;
+			if (tmp_ht && ZEND_HASH_APPLY_PROTECTION(tmp_ht)) {
+				ZEND_HASH_DEC_APPLY_COUNT(tmp_ht);
 			}
 			zval_ptr_dtor(&obj_data);
 			return;
@@ -1014,12 +1175,22 @@ void object_to_bson(zval *object, php_phongo_bson_flags_t flags, const char *key
 
 		phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE TSRMLS_CC, "Unexpected %s instance: %s", ZSTR_VAL(php_phongo_type_ce->name), ZSTR_VAL(Z_OBJCE_P(object)->name));
 		return;
-	}
+	} else {
+		HashTable *tmp_ht = HASH_OF(object);
 
-	mongoc_log(MONGOC_LOG_LEVEL_TRACE, MONGOC_LOG_DOMAIN, "encoding document");
-	bson_append_document_begin(bson, key, key_len, &child);
-	phongo_zval_to_bson(object, flags, &child, NULL TSRMLS_CC);
-	bson_append_document_end(bson, &child);
+		if (tmp_ht && ZEND_HASH_APPLY_PROTECTION(tmp_ht)) {
+			ZEND_HASH_INC_APPLY_COUNT(tmp_ht);
+		}
+
+		mongoc_log(MONGOC_LOG_LEVEL_TRACE, MONGOC_LOG_DOMAIN, "encoding document");
+		bson_append_document_begin(bson, key, key_len, &child);
+		phongo_zval_to_bson(object, flags, &child, NULL TSRMLS_CC);
+		bson_append_document_end(bson, &child);
+
+		if (tmp_ht && ZEND_HASH_APPLY_PROTECTION(tmp_ht)) {
+			ZEND_HASH_DEC_APPLY_COUNT(tmp_ht);
+		}
+	}
 }
 
 static void phongo_bson_append(bson_t *bson, php_phongo_bson_flags_t flags, const char *key, long key_len, zval *entry TSRMLS_DC)
@@ -1067,16 +1238,16 @@ try_again:
 				bson_t     child;
 				HashTable *tmp_ht = HASH_OF(entry);
 
-				if (tmp_ht) {
-					ZEND_HASH_APPLY_COUNT(tmp_ht)++;
+				if (tmp_ht && ZEND_HASH_APPLY_PROTECTION(tmp_ht)) {
+					ZEND_HASH_INC_APPLY_COUNT(tmp_ht);
 				}
 
 				bson_append_array_begin(bson, key, key_len, &child);
 				phongo_zval_to_bson(entry, flags, &child, NULL TSRMLS_CC);
 				bson_append_array_end(bson, &child);
 
-				if (tmp_ht) {
-					ZEND_HASH_APPLY_COUNT(tmp_ht)--;
+				if (tmp_ht && ZEND_HASH_APPLY_PROTECTION(tmp_ht)) {
+					ZEND_HASH_DEC_APPLY_COUNT(tmp_ht);
 				}
 				break;
 			}
@@ -1100,44 +1271,6 @@ try_again:
 	}
 }
 
-#if PHP_VERSION_ID >= 70000
-static bool is_public_property(zend_class_entry *ce, zend_string *name, zend_string **member TSRMLS_DC) /* {{{ */
-#else
-static bool is_public_property(zend_class_entry *ce, const char *prop_name, int prop_name_len TSRMLS_DC) /* {{{ */
-#endif
-{
-	zend_property_info *property_info = NULL;
-
-#if PHP_VERSION_ID >= 70000
-	if (ZSTR_VAL(name)[0] == 0) {
-		const char *prop_name,
-			 *class_name;
-		size_t prop_name_len;
-
-		zend_unmangle_property_name_ex(name,
-			&class_name, &prop_name, &prop_name_len);
-		(*member) = zend_string_init(prop_name, prop_name_len, 0);
-	} else (*member) = zend_string_copy(name);
-	property_info = zend_get_property_info(ce, (*member), 1 TSRMLS_CC);
-
-	if (!property_info) /* undefined property */
-		return true;
-
-	if (property_info == ZEND_WRONG_PROPERTY_INFO) {
-		return false;
-	}
-	
-	return (property_info->flags & ZEND_ACC_PUBLIC);
-#else
-	zval member;
-	ZVAL_STRINGL(&member, prop_name, prop_name_len, 0);
-	property_info = zend_get_property_info(ce, &member, 1 TSRMLS_CC);
-
-	return (property_info && (property_info->flags & ZEND_ACC_PUBLIC));
-#endif
-}
-/* }}} */
-
 void phongo_zval_to_bson(zval *data, php_phongo_bson_flags_t flags, bson_t *bson, bson_t **bson_out TSRMLS_DC) /* {{{ */
 {
 	HashTable   *ht_data = NULL;
@@ -1231,7 +1364,7 @@ void phongo_zval_to_bson(zval *data, php_phongo_bson_flags_t flags, bson_t *bson
 			return;
 	}
 
-	if (!ht_data || ZEND_HASH_APPLY_COUNT(ht_data) > 1) {
+	if (!ht_data || ZEND_HASH_GET_APPLY_COUNT(ht_data) > 1) {
 #if PHP_VERSION_ID >= 70000
 		if (Z_TYPE_P(data) == IS_OBJECT && instanceof_function(Z_OBJCE_P(data), php_phongo_serializable_ce TSRMLS_CC)) {
 #endif
@@ -1251,6 +1384,28 @@ void phongo_zval_to_bson(zval *data, php_phongo_bson_flags_t flags, bson_t *bson
 		zval        *value;
 
 		ZEND_HASH_FOREACH_KEY_VAL(ht_data, num_key, string_key, value) {
+			if (string_key) {
+				if (ht_data_from_properties) {
+					/* Skip protected and private properties */
+					if (ZSTR_VAL(string_key)[0] == '\0' && ZSTR_LEN(string_key) > 0) {
+						zend_string_release(string_key);
+						continue;
+					}
+				}
+
+				if (strlen(ZSTR_VAL(string_key)) != ZSTR_LEN(string_key)) {
+					phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE TSRMLS_CC, "BSON keys cannot contain null bytes. Unexpected null byte after \"%s\".", ZSTR_VAL(string_key));
+
+					return;
+				}
+
+				if (flags & PHONGO_BSON_ADD_ID) {
+					if (!strcmp(ZSTR_VAL(string_key), "_id")) {
+						flags &= ~PHONGO_BSON_ADD_ID;
+					}
+				}
+			}
+
 			/* Ensure we're working with a string key */
 			if (!string_key) {
 				string_key = zend_long_to_str(num_key);
@@ -1258,41 +1413,7 @@ void phongo_zval_to_bson(zval *data, php_phongo_bson_flags_t flags, bson_t *bson
 				zend_string_addref(string_key);
 			}
 
-			if (Z_TYPE_P(data) == IS_OBJECT) {
-				zend_string *member = NULL;
-
-				/* Ignore non-public properties */
-				if (!instanceof_function(Z_OBJCE_P(data), php_phongo_serializable_ce) &&
-					!is_public_property(Z_OBJCE_P(data), string_key, &member TSRMLS_CC)) {
-					if (member) {
-						zend_string_release(member);
-					}
-					zend_string_release(string_key);
-					continue;
-				}
-
-				if (flags & PHONGO_BSON_ADD_ID) {
-					if (!strcmp(member ? ZSTR_VAL(member) : ZSTR_VAL(string_key), "_id")) {
-						flags &= ~PHONGO_BSON_ADD_ID;
-					}
-				}
-
-				phongo_bson_append(bson, flags & ~PHONGO_BSON_ADD_ID,
-					member ? ZSTR_VAL(member) : ZSTR_VAL(string_key),
-					member ? ZSTR_LEN(member) : ZSTR_LEN(string_key),
-					value TSRMLS_CC);
-
-				if (member) {
-					zend_string_release(member);
-				}
-			} else {
-				if (flags & PHONGO_BSON_ADD_ID) {
-					if (!strcmp(ZSTR_VAL(string_key), "_id")) {
-						flags &= ~PHONGO_BSON_ADD_ID;
-					}
-				}
-				phongo_bson_append(bson, flags & ~PHONGO_BSON_ADD_ID, ZSTR_VAL(string_key), ZSTR_LEN(string_key), value TSRMLS_CC);
-			}
+			phongo_bson_append(bson, flags & ~PHONGO_BSON_ADD_ID, ZSTR_VAL(string_key), strlen(ZSTR_VAL(string_key)), value TSRMLS_CC);
 
 			zend_string_release(string_key);
 		} ZEND_HASH_FOREACH_END();
@@ -1318,17 +1439,16 @@ void phongo_zval_to_bson(zval *data, php_phongo_bson_flags_t flags, bson_t *bson
 
 		if (hash_type == HASH_KEY_IS_STRING) {
 			if (ht_data_from_properties) {
-				const char *class_name;
-				zend_unmangle_property_name(string_key, string_key_len-1, &class_name, (const char **)&string_key);
-				string_key_len = strlen(string_key);
-
-				/* Ignore non-public properties */
-				if (!is_public_property(Z_OBJCE_P(data), string_key, string_key_len TSRMLS_CC)) {
+				/* Skip protected and private properties */
+				if (string_key[0] == '\0' && string_key_len > 1) {
 					continue;
 				}
-			} else {
-				/* Chop off the \0 from string lengths */
-				string_key_len -= 1;
+			}
+
+			if (strlen(string_key) != string_key_len - 1) {
+				phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE TSRMLS_CC, "BSON keys cannot contain null bytes. Unexpected null byte after \"%s\".", ZSTR_VAL(string_key));
+
+				return;
 			}
 
 			if (flags & PHONGO_BSON_ADD_ID) {
@@ -1341,10 +1461,9 @@ void phongo_zval_to_bson(zval *data, php_phongo_bson_flags_t flags, bson_t *bson
 		/* Ensure we're working with a string key */
 		if (hash_type == HASH_KEY_IS_LONG) {
 			spprintf(&string_key, 0, "%ld", num_key);
-			string_key_len = strlen(string_key);
 		}
 
-		phongo_bson_append(bson, flags & ~PHONGO_BSON_ADD_ID, string_key, string_key_len, *value TSRMLS_CC);
+		phongo_bson_append(bson, flags & ~PHONGO_BSON_ADD_ID, string_key, strlen(string_key), *value TSRMLS_CC);
 
 		if (hash_type == HASH_KEY_IS_LONG) {
 			efree(string_key);
